#include "v3.h"


V3::V3(float x, float y, float z) {

  xyz[0] = x;
  xyz[1] = y;
  xyz[2] = z;

}
  

ostream& operator<<(ostream& os, V3 v) {

  os 
    << v.xyz[0] << " "
    << v.xyz[1] << " "
    << v.xyz[2];

  return os;

}


float& V3::operator[](int i) {

  return xyz[i];

}

V3 V3::operator+(V3 v1) {

  V3 ret;
  ret[0] = xyz[0] + v1[0];
  ret[1] = xyz[1] + v1[1];
  ret[2] = xyz[2] + v1[2];
  return ret;

}

V3 V3::operator-(V3 v1) {

  V3 ret;
  ret[0] = xyz[0] - v1[0];
  ret[1] = xyz[1] - v1[1];
  ret[2] = xyz[2] - v1[2];
  return ret;

}

float V3::operator*(V3 v1) {


  return 
    xyz[0]*v1[0] + 
    xyz[1]*v1[1] + 
    xyz[2]*v1[2];

}


V3 V3::operator/(float scf) {

  V3 ret(*this);
  ret[0] /= scf;
  ret[1] /= scf;
  ret[2] /= scf;
  return ret;

}

V3 V3::operator*(float scf) {

  V3 ret(*this);
  ret[0] *= scf;
  ret[1] *= scf;
  ret[2] *= scf;
  return ret;

}

void V3::SetFromColor(unsigned int color) {

  V3 &v = *this;
  v[0] = ((unsigned char*)&color)[0];
  v[1] = ((unsigned char*)&color)[1];
  v[2] = ((unsigned char*)&color)[2];
  v = v / 255.0f;

}

unsigned int V3::GetColor() {

  V3 &v = *this;
  unsigned int red = (int) (v[0]*255.0f+0.5f);
  unsigned int green = (int) (v[1]*255.0f+0.5f);
  unsigned int blue = (int) (v[2]*255.0f+0.5f);

  if (red < 0)
    red = 0;
  else if (red > 255)
    red = 255;
  if (green < 0)
    green = 0;
  else if (green > 255)
    green = 255;
  if (blue < 0)
    blue = 0;
  else if (blue > 255)
    blue = 255;

  unsigned int ret = 0xFF000000 + blue * 256 *256 + green * 256 + red;

  return ret;

}

// cross product
V3 V3::operator^(V3 op1) {

  V3 ret;
  ret.xyz[0] = xyz[1]*op1.xyz[2]-xyz[2]*op1.xyz[1];
  ret.xyz[1] = xyz[2]*op1.xyz[0]-xyz[0]*op1.xyz[2];
  ret.xyz[2] = xyz[0]*op1.xyz[1]-xyz[1]*op1.xyz[0];

  return ret;

}

AABB::AABB(V3 firstPoint) {

  corners[0] = corners[1] = firstPoint;

}

void AABB::AddPoint(V3 pt) {

  if (corners[0][0] > pt[0]) {
    corners[0][0] = pt[0];
  }
  if (corners[0][1] > pt[1]) {
    corners[0][1] = pt[1];
  }
  if (corners[0][2] > pt[2]) {
    corners[0][2] = pt[2];
  }


  if (corners[1][0] < pt[0]) {
    corners[1][0] = pt[0];
  }
  if (corners[1][1] < pt[1]) {
    corners[1][1] = pt[1];
  }
  if (corners[1][2] < pt[2]) {
    corners[1][2] = pt[2];
  }

}